#!/usr/bin/env python3
"""
Memory Optimization Helper for Flask App
This script provides utilities to monitor and optimize memory usage.
"""

import gc
import psutil
import logging
import threading
import time
from functools import wraps

# Configure logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

class MemoryMonitor:
    def __init__(self, threshold_mb=500, check_interval=60):
        self.threshold_mb = threshold_mb
        self.check_interval = check_interval
        self.monitoring = False
        self.monitor_thread = None
        
    def get_memory_usage(self):
        """Get current memory usage in MB"""
        process = psutil.Process()
        return process.memory_info().rss / 1024 / 1024
    
    def force_garbage_collection(self):
        """Force garbage collection and clear matplotlib cache"""
        try:
            # Force garbage collection
            collected = gc.collect()
            
            # Clear matplotlib cache if available
            try:
                import matplotlib.pyplot as plt
                plt.close('all')
                # Clear matplotlib font cache
                if hasattr(plt, '_get_cachedir'):
                    import matplotlib as mpl
                    mpl.font_manager._get_fontconfig_fonts.cache_clear()
            except:
                pass
            
            logger.info(f"Garbage collection freed {collected} objects")
            return collected
        except Exception as e:
            logger.error(f"Error during garbage collection: {e}")
            return 0
    
    def check_memory(self):
        """Check memory usage and clean up if necessary"""
        current_mb = self.get_memory_usage()
        
        if current_mb > self.threshold_mb:
            logger.warning(f"Memory usage ({current_mb:.1f}MB) exceeds threshold ({self.threshold_mb}MB)")
            freed = self.force_garbage_collection()
            new_mb = self.get_memory_usage()
            logger.info(f"Memory after cleanup: {new_mb:.1f}MB (freed {current_mb - new_mb:.1f}MB)")
            return True
        
        return False
    
    def start_monitoring(self):
        """Start memory monitoring in a separate thread"""
        if self.monitoring:
            logger.warning("Memory monitoring already running")
            return
        
        self.monitoring = True
        self.monitor_thread = threading.Thread(target=self._monitor_loop, daemon=True)
        self.monitor_thread.start()
        logger.info(f"Memory monitoring started (threshold: {self.threshold_mb}MB)")
    
    def stop_monitoring(self):
        """Stop memory monitoring"""
        self.monitoring = False
        if self.monitor_thread:
            self.monitor_thread.join()
        logger.info("Memory monitoring stopped")
    
    def _monitor_loop(self):
        """Internal monitoring loop"""
        while self.monitoring:
            try:
                self.check_memory()
                time.sleep(self.check_interval)
            except Exception as e:
                logger.error(f"Error in memory monitoring loop: {e}")
                time.sleep(self.check_interval)

# Global memory monitor instance
memory_monitor = MemoryMonitor()

def memory_cleanup_decorator(func):
    """Decorator to clean up memory after function execution"""
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            result = func(*args, **kwargs)
            return result
        finally:
            # Clean up matplotlib figures
            try:
                import matplotlib.pyplot as plt
                plt.close('all')
            except:
                pass
            
            # Force garbage collection if memory is high
            if memory_monitor.get_memory_usage() > 300:  # 300MB threshold
                memory_monitor.force_garbage_collection()
    
    return wrapper

def optimize_matplotlib():
    """Optimize matplotlib settings for memory efficiency"""
    try:
        import matplotlib
        import matplotlib.pyplot as plt
        
        # Set non-interactive backend
        matplotlib.use('Agg')
        
        # Configure matplotlib for memory efficiency
        plt.rcParams['figure.max_open_warning'] = 10
        plt.rcParams['axes.formatter.limits'] = [-3, 3]
        
        # Reduce DPI for memory savings (you can adjust this)
        plt.rcParams['figure.dpi'] = 80
        plt.rcParams['savefig.dpi'] = 80
        
        logger.info("Matplotlib optimized for memory efficiency")
        return True
    except Exception as e:
        logger.error(f"Error optimizing matplotlib: {e}")
        return False

def get_system_info():
    """Get system memory information"""
    try:
        memory = psutil.virtual_memory()
        process = psutil.Process()
        
        info = {
            'total_memory_gb': memory.total / (1024**3),
            'available_memory_gb': memory.available / (1024**3),
            'memory_percent': memory.percent,
            'process_memory_mb': process.memory_info().rss / (1024**2),
            'process_memory_percent': process.memory_percent()
        }
        
        return info
    except Exception as e:
        logger.error(f"Error getting system info: {e}")
        return {}

def print_memory_stats():
    """Print current memory statistics"""
    info = get_system_info()
    if info:
        print(f"System Memory: {info['total_memory_gb']:.1f}GB total, "
              f"{info['available_memory_gb']:.1f}GB available "
              f"({info['memory_percent']:.1f}% used)")
        print(f"Process Memory: {info['process_memory_mb']:.1f}MB "
              f"({info['process_memory_percent']:.1f}% of system)")

if __name__ == '__main__':
    print("Memory Optimization Helper")
    print("=" * 40)
    
    # Show current memory stats
    print_memory_stats()
    
    # Test memory cleanup
    print("\nTesting memory cleanup...")
    before_mb = memory_monitor.get_memory_usage()
    freed = memory_monitor.force_garbage_collection()
    after_mb = memory_monitor.get_memory_usage()
    
    print(f"Memory before cleanup: {before_mb:.1f}MB")
    print(f"Memory after cleanup: {after_mb:.1f}MB")
    print(f"Objects freed: {freed}")
    
    # Optimize matplotlib
    print("\nOptimizing matplotlib...")
    optimize_matplotlib()
    
    print("\nMemory optimization complete!")